function [phi, resid, rssq] =  setarestim(y, thres, d, p)
%    Threshold autoregressive (TAR) model estimation
%    y= time series data, column vector
%    thres =  threshold values, including interval extremes
%    d  =  delay parameter
%    p  =  autoregressive order
% --- notice thres: thres(1)=value less than the smallest observation, 
%       thres(nr+1)=largest observation (the program check for <=)
%    OUTPUT parameter phi each row is an AR parameter sequence, rows are
%    ordered state regime (linear models for each state regime)
%    RESIDUALS in output u are a column vector
%    rssq  is the RESIDUAL SUM OF SQUARES, not the residual variance
n = size(y, 1);
nr1 = size(thres, 2); nr = nr1-1; %    nr = number of regimes  
m =  max(p, d); % observations lost at the beginning of the time series
Z = y(m+1:n); % dependent variable
ZS = y(m-d+1:n-d); %  state variable
ZMAT = ones(n-m, p+1); % initialize the independent variables
phi =  zeros(nr, p+1);  %  initialize the parameter array
for ic = 1:p, ZMAT(:,ic+1) = y(m-ic+1:n-ic); end, %ZMAT, % compute the independet var
for  regime = 1:nr, % for each regime performs least squares estimation
    clear r1,  clear r2, r1 = thres(regime);  r2 = thres(regime+1);  %  r2=threshold
    clear idx, idx  =  find(ZS>r1 & ZS<=r2); % index of the observations in the regime
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
    phi(regime,:) = (XMAT\X)';  % lesast squraes solution of equation XMAT*phi=X
end,
%phi, 
resid = zeros(n,1); % initialize the residual column vector
for  regime = 1:nr, % for each regime compute the residuals
    clear r1,  clear r2, r1 = thres(regime);  r2 = thres(regime+1);  %  r2=threshold
    clear idx, idx  =  find(ZS>r1 & ZS<=r2); %qqx=idx+m,% index of the observations in the regime
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
    clear th, th=(phi(regime,:))'; resid(idx+m)=X-XMAT*th;  % residuals in regime with y*=XMAT*th
end,
%rssq = norm(resid), %  norm computes the square root of the sum of squares 
rssq  =  resid'*resid;   %  the residual sum of squares 
%   setar40estim  program  ends